SystemOrganization addCategory: #ManifestoBrowser!

TestCase subclass: #LintRuleTreeTest
	instanceVariableNames: 'head'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!LintRuleTreeTest methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 14:24'!
createTreeA
	| l1 l2 n0 l3 l4 l5 n1 n2 n3 |
	l1 := LintRuleLeafNode forRule: RBAsOrderedCollectionNotNeededRule new.
	l2 := LintRuleLeafNode forRule: RBYourselfNotUsedRule new.
	l3 := LintRuleLeafNode forRule: RBExcessiveInheritanceRule new.
	l4 := LintRuleLeafNode forRule: RBExcessiveArgumentsRule new.
	l5 := LintRuleLeafNode forRule: RBExcessiveMethodsRule new.
	
	n1 := LintRuleCompNode new: '1'.
	n1 addChild: l1; addChild: l2.
	
	n2 := LintRuleCompNode new: '2'.
	n2 addChild: l3.
	
	n3 := LintRuleCompNode new: '3'.
	n3 addChild: l4; addChild: l5.
	n2 addChild: n3.
	
	n0 := LintRuleCompNode new: '0'.
	
	n0 addChild: n1; addChild: n2.
	
	
	^LintRuleTree new head: n0! !

!LintRuleTreeTest methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 19:29'!
createTreeB
	| l1 l2 n0 l3 l4 l5 n1 n2 n3 |
	l4 := LintRuleLeafNode forRule: RBExcessiveArgumentsRule new.
	l5 := LintRuleLeafNode forRule: RBExcessiveMethodsRule new.
	
	n2 := LintRuleCompNode new: '2'.
	
	n3 := LintRuleCompNode new: '3'.
	n3 addChild: l4; addChild: l5.
	n2 addChild: n3.
	
	n0 := LintRuleCompNode new: '0'.
	
	n0 addChild: n2.
	
	^LintRuleTree new head: n0! !

!LintRuleTreeTest methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 14:24'!
testRemoveLeafNodes
	| tree1 tree2 |
	tree1 := self createTreeA.
	tree2 := self createTreeB.
	
	"(LintRuleBrowser for: tree1) open."
	
	tree1 removeNamed: RBAsOrderedCollectionNotNeededRule new name.
	tree1 removeNamed: RBYourselfNotUsedRule new name.
	tree1 removeNamed: RBExcessiveInheritanceRule new name.
	
	"(LintRuleBrowser for: tree1) open.
	(LintRuleBrowser for: tree2) open."
	
	self assert: (tree1 = tree2)! !

TestCase subclass: #ManifestoBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!ManifestoBrowserTest methodsFor: 'util' stamp: 'AlexandreBergel 11/4/2011 15:38'!
createPackageNamed: aByteString 
	^ PackageFacade createPackageNamed: aByteString! !

!ManifestoBrowserTest methodsFor: 'util' stamp: 'AngelNunez 9/28/2011 14:27'!
packageName
	^ 'ManifestoBrowser'! !

!ManifestoBrowserTest methodsFor: 'util' stamp: 'AlexandreBergel 11/4/2011 15:56'!
removePackage: aPackage
	PackageFacade removePackage: aPackage! !

!ManifestoBrowserTest methodsFor: 'tests' stamp: 'AlexandreBergel 11/4/2011 15:56'!
testCreateManisfesto

	| aPackage manifestoClass |
	aPackage := self createPackageNamed: 'FooBar'.
	self assert: (ManifestoBrowser hasManifesto: aPackage) not.
	
	ManifestoBrowser createManifesto: aPackage.
	self assert: (ManifestoBrowser hasManifesto: aPackage).
	
	manifestoClass := ManifestoBrowser manifestoOf: aPackage.
	self assert: manifestoClass notNil.
	
	"Now we clean"
	manifestoClass removeFromSystem.
	self removePackage: aPackage
	! !

!ManifestoBrowserTest methodsFor: 'tests' stamp: 'AlexandreBergel 11/4/2011 16:10'!
testManisfestoName

	| aPackage |
	aPackage := self createPackageNamed: 'FooBar'.
	self assert: (ManifestoBrowser manifestoNameOf: aPackage) = 'ManifestoFooBar'.
	self removePackage: aPackage.
	
	aPackage := self createPackageNamed: 'Foo-Bar-Zork'.
	self assert: (ManifestoBrowser manifestoNameOf: aPackage) = 'ManifestoFooBarZork'.
	self removePackage: aPackage.
! !

Object subclass: #LintRuleBrowser
	instanceVariableNames: 'tree browser environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!LintRuleBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 18:30'!
for: tree
	^self for:tree onEnvironment: (BrowserEnvironment new forPackages: (Array new))! !

!LintRuleBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/4/2011 12:38'!
for: tree onEnvironment: env
	^self new tree:tree environment: env! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/21/2011 09:13'!
addNodes: nodes
	tree addNodes: nodes! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:25'!
formatTree: a
	^a tree
			title: 'Lint Rules'; 
			children: [ :node :i | node isLeaf ifTrue: [OrderedCollection new] ifFalse: [ node children ] ];
			rootsExpanded; 
			format: [ :node | self nodeDisplayName: node ];
			icon: [:node | self nodeDisplayIcon: node ];
			beMultiple;
			
			act: [:p | p selection ifNotNilDo: [:nodes | self runLint: nodes ]] 
				icon: UITheme current smallForwardIcon  entitled: 'Run rules';
				
			act: [:p | p selection ifNotNilDo: [:nodes | self removeNodes: nodes ]] 
				icon: UITheme current smallDeleteIcon entitled: 'Remove';
				
			act: [:p | self openAddRuleDialog ] 
				icon: UITheme current smallRightFlushIcon entitled: 'Add rules'! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 17:14'!
initialTree
	^ LintRuleTree for: RBCompositeLintRule lintChecks! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/10/2011 10:43'!
nodeDisplayIcon: node
	^node analized 
		ifTrue: [ node allAnalized 
						ifTrue: [ node problemCount = 0 
										ifTrue: UITheme current smallOkIcon  
										ifFalse: UITheme current smallErrorIcon ]
						ifFalse: [ node problemCount > 0 ifTrue: [UITheme current smallErrorIcon] ]]! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/5/2011 15:51'!
nodeDisplayName: node
	^node name , ' ', 
	(node analized 
			ifTrue: '(',node problemCount printString, (node allAnalized ifTrue:[''] ifFalse:[' partially']),')' 
			ifFalse: '')! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 14:00'!
open
	"(LintRuleBrowser for:(LintRuleTree for: RBCompositeLintRule lintChecks) onEnvironment: (BrowserEnvironment new forPackageNames: #('Mondrian-Core'))) open"

	self openOn: tree head children! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 17:20'!
openAddRuleDialog
	self openAddRuleDialogOn: (self initialTree removeNodesIn: tree) head children! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/21/2011 09:13'!
openAddRuleDialogOn: nodes

	| finder validator |
	finder := GLMFinder new.
	validator := GLMValidator new.
	validator
		show: [ :a | 
			a tree
				title: 'Lint-Check Rules';
				beMultiple;
				children: [ :node :i | 
							node isLeaf
								ifTrue: [ OrderedCollection new ]
								ifFalse: [ node children ] ];
				rootsExpanded;
				format: [ :node | node name ] ].
	validator
		validator: 'Ok'
		act: [ :x | 
			x selection ifNotNilDo: [:nds | self addNodes: nds ].
			x close.
			browser update ].
	finder show: [ :a | a custom: validator ].
	finder openOn: nodes! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 17:24'!
openOn: nodes

	browser := GLMTabulator new.	
	browser title: self title.
	browser 
		column: #rules;
		column: #preview. 
		
	browser transmit to: #rules; andShow: [ :a | 
		self formatTree: a].
		      
	browser transmit to: #preview; from: #rules; andShow: [ :a |
		a text
			title: 'Results';
			when: [:x| (x size = 1) and: ((x atWrap:1) isLeaf )];
			display: [ :x | ((x atWrap:1) rule asNode update: (BrowserEnvironment new)) text];
			act: [:x | (x entity atWrap: 1) rule class browse ] icon: UITheme current smallInspectItIcon entitled: 'Inspect'
			 ].
		
	browser openOn: nodes! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:27'!
removeNodes: nodes
	tree removeNodes: nodes.
	browser update! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:28'!
runLint: selectedNodes
	SmalllintChecker 
		runRule: (RBCompositeLintRule rules: (selectedNodes collect:[:each | each toRule ])) 
		onEnvironment: environment.
	selectedNodes do: [:each | each analized:true]. 
	browser update! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/4/2011 16:13'!
title
	^'Lint Browser for ',(environment packages collect:[:each | each packageName ]) asCommaString! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 9/30/2011 21:21'!
tree
	^tree! !

!LintRuleBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/10/2011 10:42'!
tree:t environment: env
	tree:= t.
	environment  := env! !

LintRuleBrowser subclass: #LintRulePackageBrowser
	instanceVariableNames: 'package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!LintRulePackageBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/21/2011 09:14'!
addNodes: nodes
	ManifestoBrowser 
		openCreateManifestoWarning: package name 
		onOkDo: [super addNodes: nodes.
				self saveTree]! !

!LintRulePackageBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/21/2011 09:11'!
formatTree: a
	^(super formatTree: a) 
		"act: [:p | ManifestoBrowser openCreateManifestoWarning: package name onOkDo: [self saveTree] ] 
			icon: UITheme current smallSaveIcon  entitled: 'Save tree'
		)"
		! !

!LintRulePackageBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 14:16'!
getTree
	(ManifestoBrowser manifestoOf: package) rules
		ifNotNilDo: [ :storedRuleClassNames | 
			| leaves aTree |
			aTree := LintRuleTree for: RBCompositeLintRule lintChecks.
			
			leaves := aTree head leaves.
			leaves
				removeAllSuchThat: [ :leaf | 
					(storedRuleClassNames findFirst: [ :className | className = leaf rule class name ]) > 0 ].
			
			aTree removeLeaves: leaves.
			^ aTree 
		].
	^nil
	! !

!LintRulePackageBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 14:13'!
init: aPackage
	| aTree |
	
	package := aPackage.
	
	aTree := (ManifestoBrowser hasManifesto: package) ifTrue: [self getTree].		
	aTree ifNil: [ aTree := LintRuleTree for: RBCompositeLintRule lintChecks ].
	
	^super tree: aTree environment: (BrowserEnvironment new forPackages: (Array with: package))! !

!LintRulePackageBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/21/2011 09:10'!
removeNodes: nodes
	ManifestoBrowser 
		openCreateManifestoWarning: package name 
		onOkDo: [super removeNodes: nodes.
				self saveTree]! !

!LintRulePackageBrowser methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/21/2011 09:09'!
saveTree
	
	(ManifestoBrowser hasManifesto: package)
		ifFalse: [ ManifestoBrowser createManifesto: package ].

	(ManifestoBrowser manifestoOf: package) class removeSelector:  #rules.
	(ManifestoBrowser manifestoOf: package) class compile: 'rules 
				^', ((tree head leaves collect:[:each | each rule class name]) storeString) 
	! !

Object subclass: #LintRuleNode
	instanceVariableNames: 'name analized parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

LintRuleNode subclass: #LintRuleCompNode
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!LintRuleCompNode class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 19:27'!
forRule: rule
	| node |
	node := LintRuleCompNode new: rule name.
	rule rules do: [:each | node addChild: (LintRuleNode forRule: each)].
	^ node.! !

!LintRuleCompNode class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 18:25'!
new: aName
	^self new name: aName! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 19:11'!
= node
	(super = node) 
		ifTrue: [
			node isLeaf 
				ifFalse: [
					^(node children = children)
				] 
		].
	^false 
	! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 15:37'!
addChild: node
	children ifNil: [children := LinkedList new].
	children add: node.
	node parent: self! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:42'!
allAnalized
	children do:[: each | each analized ifFalse: [^false]].
	^true  ! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:42'!
allRules
	| allRules |
	allRules := LinkedList new. 
      children do: [:each| each allRules do: [:each2 | allRules add: each2]].
	^allRules! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:17'!
analized
	children do:[: each | each analized ifTrue: [^true]]. 
	^false ! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:20'!
analized: v
	children do:[:each | each analized:v]! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:06'!
children
	^children! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:06'!
children: c
	children := c! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 11:54'!
descendantNamed: aName
	(super descendantNamed: aName) 
		ifNil: [
			self children do: [:each | (each descendantNamed: aName) ifNotNilDo: [:v | ^v ]].
			^nil. 
			] 
		ifNotNilDo: [:v | ^v]. 
	! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:22'!
isLeaf
	^false! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:40'!
leaves
	| leaves |
	leaves := LinkedList new.
	self children do:[:each | each leaves do:[:each2| leaves add: each2]].
	^leaves! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:25'!
problemCount
	| count |
	count := 0. 
	children do: [:each | count := count + each problemCount]. 
	^count! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:27'!
remove: node
	children remove: node ifAbsent: [ children do: [:each | each remove: node]].
	children isEmpty ifTrue:[parent ifNotNilDo: [:p | p remove:self]]! !

!LintRuleCompNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:28'!
removeNamed:  aName
	children removeAllSuchThat: [:each | (each name compare: aName) = 2]. 
	children do: [:each | each removeNamed: aName].
	children isEmpty ifTrue:[parent ifNotNilDo: [:p | p remove:self]]! !

LintRuleNode subclass: #LintRuleLeafNode
	instanceVariableNames: 'rule'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!LintRuleLeafNode class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 18:26'!
forRule: rule
	| node |
	node := LintRuleLeafNode new name: rule name.
	node rule: rule.
	^ node.! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:43'!
allAnalized
	^self analized! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 14:19'!
allRules
	^Array with: self rule! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:42'!
analized
	^analized isNil ifTrue: [false] ifFalse: [analized]! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:20'!
analized: v
	analized := v! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:22'!
isLeaf
	^true! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:41'!
leaves
	| leaves |
	leaves := LinkedList new.
	leaves add: self.
	^leaves! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:25'!
problemCount
	^self rule problemCount! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 13:58'!
rule
	^rule! !

!LintRuleLeafNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 13:58'!
rule: r
	rule := r! !

!LintRuleNode class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 19:26'!
forRule: rule
	^rule isComposite ifTrue: [LintRuleCompNode forRule: rule] ifFalse:[LintRuleLeafNode forRule: rule] ! !

!LintRuleNode class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 18:24'!
name: aName
	name := aName
	! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:37'!
= node
	^name = node name! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:42'!
allAnalized
! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:39'!
allRules
! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:16'!
analized
	! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:20'!
analized: v
! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:39'!
descendantNamed: aName
	^(self name = aName) ifTrue: [self] ifFalse: [nil]! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:21'!
isLeaf! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:40'!
leaves! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 9/30/2011 21:14'!
name
	^name! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/10/2011 10:41'!
name: s
	name := s! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/1/2011 09:16'!
parent
	^parent! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 9/30/2011 23:48'!
parent: node
	parent := node! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:23'!
problemCount! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:27'!
remove: node! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:28'!
removeNamed:  aName
! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:41'!
root
	^self parent ifNil: [self ] ifNotNilDo: [:p | p root] ! !

!LintRuleNode methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 15:43'!
toRule
	^ RBCompositeLintRule rules: self allRules! !

Object subclass: #LintRuleTree
	instanceVariableNames: 'head'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!LintRuleTree class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 16:21'!
for: rule
	"LintRuleTree for: RBCompositeLintRule lintChecks"

	| node |
	node := LintRuleCompNode new: 'Dummy'.
	node addChild: (LintRuleNode forRule: rule).
	^self new head: node! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 19:10'!
= tree
	^head = tree head! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 15:41'!
addNode: node
	(self searchNamed: node parent name) 
					ifNil: [ | parent | 
						parent := LintRuleCompNode new name: node parent name.
		 				parent addChild: node.
						self addNode: parent
						]
					ifNotNilDo: [:n | n addChild: node]! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 17:26'!
addNodes: nodes
	(self leaves: nodes)
		do: [:leaf |
				(self searchNamed: leaf parent name) 
					ifNil: [ | n parent | 
						n := (self searchNamed: leaf parent parent name).
		 				parent := LintRuleCompNode new name: leaf parent name.
		 				parent addChild: leaf.
						n addChild: parent ]
					ifNotNilDo: [:n | n addChild: leaf] ]! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 9/30/2011 21:14'!
head
	^head! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 9/30/2011 21:13'!
head: node
	head := node! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/1/2011 09:58'!
leaves
	^head leaves! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 14:34'!
leaves: nodes
	| list |
	list := LinkedList new.
	nodes do:[:node | node leaves do:[:leaf | list add: leaf]].
	^list! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 9/30/2011 21:47'!
remove: node
	head remove: node! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 16:46'!
removeLeaves: leaves
	leaves do: [ :leaf | head removeNamed: leaf name ]! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/5/2011 15:33'!
removeNamed: aName
	head removeNamed: aName! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:28'!
removeNodes: nodes
	nodes do: [:node | self remove: node]! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 17:27'!
removeNodesIn: aTree
	(aTree leaves) do:[:each| self removeNamed: each name]
	! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/18/2011 17:28'!
searchNamed: name
	^self searchNamed: name in:head! !

!LintRuleTree methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 11:43'!
searchNamed: name in: node
	| v |
	(node name = name)
		ifTrue: [^node] 
		ifFalse: [ node isLeaf 
			ifFalse: [node children do: [:each | v := self searchNamed: name in: each. v ifNotNilDo: [:n|^v]]]].
	^nil. ! !

Object subclass: #ManifestoBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 11/4/2011 16:14'!
createManifesto: package
	| cls newName |
	
	newName := ManifestoBrowser manifestoNameOf: package .
	cls := Object subclass: newName asSymbol 
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: package packageName asString.

	PackageFacade registerClass: cls inPackage: package.
	
	cls class instanceVariableNames: ''.
	cls class compile: 'rules
		^nil'.
	cls class compile: 'packageDescription
					^'''''.! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/17/2011 16:22'!
hasManifesto: aPackage
	"this should be an extension method dfined on the class RPackage"
	
	^ aPackage classes anySatisfy: [ :aClass | aClass name = (self manifestoNameOf: aPackage) ]! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/7/2011 14:58'!
icon
	^UITheme current smallForwardIcon! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 11/4/2011 16:13'!
manifestoNameOf: aPackage

	^ 'Manifesto', (aPackage packageName copyWithout: $-)! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 15:11'!
manifestoNameOfPackageNamed: aPackageName

	^ 'Manifesto', (aPackageName copyWithout: $-)! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/6/2011 15:36'!
manifestoOf: aPackage

	^ Compiler evaluate: (self manifestoNameOf: aPackage)! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/7/2011 14:57'!
menuCommandOn: aBuilder
	<worldMenu>

	(aBuilder item: #'ManifestoBrowser')
		order: 6; 
		parent: #Tools;
		label: 'Manifesto Browser';
		icon: self icon;
		action:[ManifestoBrowser open]! !

!ManifestoBrowser class methodsFor: 'public' stamp: 'AngelNunez 9/28/2011 13:43'!
open
	"Open a browser"
	^ self new open! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 11/4/2011 16:03'!
openCreateManifestoWarning: aPackageName onOkDo: aBlock
	"not so nice yet"
	
	(ManifestoBrowser hasManifesto: (ManifestoBrowser forPackageNamed: aPackageName)) 
		ifTrue: aBlock  
		ifFalse: [
			self openWarning: 'A class ''', 
				(ManifestoBrowser manifestoNameOfPackageNamed: aPackageName), 
				''' will be created in the package ''',  
				aPackageName,''''
			onOkDo: aBlock]! !

!ManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AngelNunez 10/19/2011 15:16'!
openWarning: aText onOkDo: aBlock

	| validator |
	validator := GLMValidator new.
	validator
		show: [ :a | 
			a text
				display: [:x | x]].
	validator
		validator: 'Ok'
		act: [ :x | aBlock value.
			x close.].
	validator openOn: aText! !

!ManifestoBrowser methodsFor: 'accessing' stamp: 'AlexandreBergel 11/4/2011 16:03'!
authors: packageName
	^((ManifestoBrowser forPackageNamed: packageName) methods  
		collect: [ :cm| 
			| s | 
			s := cm timeStamp substrings. 
			s isEmpty ifTrue:[''] ifFalse:[ s first ]
			]) asSet! !

!ManifestoBrowser methodsFor: 'accessing' stamp: 'AlexandreBergel 11/4/2011 16:03'!
description: packageName
	| package | 
	
	package := ManifestoBrowser forPackageNamed: packageName.
	
	^(ManifestoBrowser hasManifesto: package) 
		ifTrue: [(ManifestoBrowser manifestoOf: package) packageDescription ifNil: [''] ifNotNilDo:[:d|d] ] 
		ifFalse:['<<write a description for the package here>>']! !

!ManifestoBrowser methodsFor: 'opening' stamp: 'AngelNunez 10/21/2011 10:08'!
open
	"self new open"
	^ self openOn: (self sortedPackages collect: [ :each | each name ])! !

!ManifestoBrowser methodsFor: 'opening' stamp: 'AlexandreBergel 11/4/2011 16:02'!
openLintBrowser: x
	 x selection 
		ifNotNilDo: 
			[ :packageName | (LintRulePackageBrowser forPackageNamed: packageName) open]! !

!ManifestoBrowser methodsFor: 'opening' stamp: 'AngelNunez 10/21/2011 10:12'!
openOn: aPackageNameList

	| browser |
	browser := GLMTabulator new.
	browser title: 'Manifesto Browser'.
	browser
		column: #packages;
		column: #manifesto.
		
	browser transmit
		to: #packages;
		andShow: [ :a | 
					a list title: 'Package';
						beSingle;
						act: [ :x | self openLintBrowser: x ]
							icon: UITheme current smallForwardIcon entitled: 'Open Lint Browser' ].
						
	browser transmit
		to: #manifesto;
		from: #packages;
		andShow: [ :a | 
					a stackedVerticallyArrangement.
					a list title: 'Authors';
						display: [ :x | self authors: x ].
						
					a text title: 'Description';
						display: [ :x | self description: x ];
						"
						act: [ :x | self saveDescription: x ] 
							icon: UITheme current smallSaveIcon entitled: 'Save description';
						"				
						act: [ :x | self saveDescription: x.  ] 
							on: $s entitled: 'Save description'
						].
											
	browser openOn: aPackageNameList! !

!ManifestoBrowser methodsFor: 'opening' stamp: 'AngelNunez 10/21/2011 10:01'!
saveDescription: x
	^ ManifestoBrowser
		openCreateManifestoWarning: x entity
		onOkDo: [ self saveDescription: x entity description: x text ]! !

!ManifestoBrowser methodsFor: 'saving' stamp: 'AlexandreBergel 11/4/2011 16:03'!
saveDescription: packageName description: description
	| package manifesto |
	
	package := ManifestoBrowser forPackageNamed: packageName.
	
	(ManifestoBrowser hasManifesto: package)
		ifFalse: [ ManifestoBrowser createManifesto: package ].
		
	manifesto := ManifestoBrowser manifestoOf: package.
	
	manifesto class removeSelector: #packageDescription.
	manifesto class compile:
			'packageDescription 
		^''' , description , ''''! !

!ManifestoBrowser methodsFor: 'util' stamp: 'AlexandreBergel 11/4/2011 16:07'!
sortedPackages
	^ PackageFacade sortedPackages! !

Object subclass: #ManifestoManifestoBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!ManifestoManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 11/4/2011 15:31'!
packageDescription
					^''! !

!ManifestoManifestoBrowser class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 11/4/2011 15:31'!
rules
		^nil! !

Object subclass: #PackageFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ManifestoBrowser'!

!PackageFacade commentStamp: 'AlexandreBergel 11/4/2011 15:34' prior: 0!
To make the manifesto browser works with RPackage and PackageInfo, we use this facade!

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 16:07'!
createPackageNamed: aPackageName
	"Return a new package"

	| cls p |
	"We use RPackage if installed"
	self hasRPackage
		ifTrue: [ 
			^ self organizerClass default
				packageNamed: aPackageName asSymbol
				ifAbsent: [ 
					"We create and register the new package"
					p := self packageClass named: aPackageName asString.
					self organizerClass default registerPackage: p.
					p ] ].	"If not, then we use PackageInfo"
	^ self packageClass named: aPackageName asString! !

!PackageFacade class methodsFor: 'testing' stamp: 'AlexandreBergel 11/4/2011 16:01'!
hasRPackage
	^ Smalltalk includesKey: #RPackage! !

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 16:06'!
organizerClass
	"Return the organizer package class"
	
	^ self hasRPackage 
		ifTrue: [ (Smalltalk at: #RPackageOrganizer )]
		ifFalse: [ (Smalltalk at: #PackageOrganizer )]! !

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 16:06'!
packageClass
	"Return the organizer package class"
	
	^ self hasRPackage 
		ifTrue: [ (Smalltalk at: #RPackage )]
		ifFalse: [ (Smalltalk at: #PackageInfo )]! !

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 16:01'!
packageNamed: packageName
	self hasRPackage 
		ifTrue: [ ^ (Smalltalk at: #RPackageOrganizer) default packageNamed: packageName ].
	
	^ PackageOrganizer default packageNamed: packageName! !

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 15:48'!
registerClass: cls inPackage: package
	"Register a class in a package. This is useful only for RPackage. We do nothing for PackageInfo"
	
	"(package class name == #RPackage)
		ifTrue: [ package addClassDefinition: cls ]"! !

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 15:58'!
removePackage: aPackage 
	"With RPackage"
	(aPackage class name == #RPackage)
		ifTrue: [ (Smalltalk at: #RPackageOrganizer) default unregisterPackage: aPackage. ^ self ].
		
	"We package info"
	(Smalltalk at: #PackageOrganizer) default unregisterPackage: aPackage! !

!PackageFacade class methodsFor: 'managing packages' stamp: 'AlexandreBergel 11/4/2011 16:05'!
sortedPackages

	self hasRPackage 
		ifTrue: [ ^ (Smalltalk at: #RPackageOrganizer) default packages 
					asSortedCollection: [ :a :b | a name <= b name ] ].
				
	^ (Smalltalk at: #PackageOrganizer) default  packages 
					asSortedCollection: [ :a :b | a name <= b name ]
				! !
